import { ChainTask, ChainTaskClass } from 'models/ChainTask'
import { RequestTask } from 'models/RequestTask'
import { ChainQueue } from 'queue/chain.queue'
import { DocumentType } from '@typegoose/typegoose'
import { BlockChain } from 'chain/BlockChain'
import { BaseController, role, router, ZError } from 'zutils'

class ChainController extends BaseController {
  @role('anon')
  @router('post /chain/req')
  async addChainRequest(req, res) {
    // data是一个数组!!
    const { data, source, cb, taskId, force, max } = req.params
    console.log(`income chain request:: taskId: ${taskId}, source: ${source}, cb: ${cb}`)
    console.log(JSON.stringify(data))
    let maxTryCount = max !== undefined ? parseInt(max) : parseInt(process.env.CHAIN_MAX_TRY)
    let chainTask: DocumentType<ChainTaskClass> = await ChainTask.insertOrUpdate(
      { taskId },
      { source, taskData: data, cb },
    )

    if ((!chainTask.newRecord && force) || chainTask.newRecord) {
      for (let sub of data) {
        let subType = sub.type
        let subTask = new RequestTask({
          taskId,
          chainTaskId: chainTask.id,
          taskType: subType,
          reqData: sub,
          maxTryCount,
        })
        await subTask.save()
        chainTask.tasks.pushOnce(subTask.id)
        new ChainQueue().addTaskToQueue(subTask)
      }
    }
    chainTask.newRecord = false
    await chainTask.save()
    return chainTask.toJson()
  }

  @role('anon')
  @router('post /chain/query_info')
  async queryUserInfo(req, res) {
    let { address } = req.params
    if (!address) {
      throw new ZError(10, 'address is required')
    }
    let info = await new BlockChain().distributorReactor.getMintableCount({ user: address })
    return { count: info }
  }
}
export default ChainController
