import { CheckIn } from 'models/CheckIn'
import { NftHolder } from 'models/NftHolder'
import { BaseController, role, router, ZError } from 'zutils'
import { getMonthBegin, getNDayAgo } from 'utils/utcdate.util'

class TaskController extends BaseController {
  @role('anon')
  @router('post /task/check_in')
  async checkDailyCheckIn(req, res) {
    let { address, days, limit } = req.params
    if (!address || (!days && !limit)) {
      throw new ZError(10, 'params mismatch')
    }
    let query: any = { from: address.toLowerCase() }
    if (!limit) {
      if (typeof days === 'number') {
        let begin = getNDayAgo(days, true)
        query.blockTime = { $gt: (begin.getTime() / 1000) | 0 }
      } else if (typeof days === 'string') {
        if (days === '1month') {
          let date = getMonthBegin(new Date())
          query.blockTime = { $gt: (date.getTime() / 1000) | 0 }
        } else {
          query.dateTag = days
        }
      } else if (Array.isArray(days)) {
        query.dateTag = { $in: days }
      }
    }
    let records
    if (limit) {
      records = await CheckIn.find(query).sort({ _id: -1 }).limit(limit)
    } else {
      records = await CheckIn.find(query).sort({ _id: -1 })
    }
    let result = []
    for (let record of records) {
      result.push(record.toJson())
    }
    return result
  }

  @role('anon')
  @router('post /task/check_in/max_seq')
  async maxSeqCheckin(req, res) {
    let { address } = req.params
    if (!address) {
      throw new ZError(10, 'params mismatch')
    }
    const record = await CheckIn.findOne({ from: address.toLowerCase() }).sort({ count: -1 })
    return record.toJson()
  }

  @role('anon')
  @router('post /task/nft/checkburn')
  async checkNFTBurn(req, res) {
    let { address, chain, user } = req.params
    if (!address || !chain || !user) {
      throw new ZError(10, 'params mismatch')
    }
    address = address.toLowerCase()
    user = user.toLowerCase()

    let records = await NftHolder.find({ address, chain, user, burn: true }).sort({ blockNumber: -1 })
    let result = []
    for (let record of records) {
      result.push({
        address: record.address,
        chain: record.chain,
        user: record.user,
        tokenId: record.tokenId,
      })
    }
    return result
  }
}
export default TaskController
