import * as dotenv from 'dotenv'
import logger from 'logger/logger'
const envFile = process.env.NODE_ENV && process.env.NODE_ENV === 'production' ? `.env.production` : '.env.development'
dotenv.config({ path: envFile })

const listFile =
  process.env.NODE_ENV && process.env.NODE_ENV === 'production'
    ? `../config/event_list_production.json`
    : '../config/event_list.json'
const events = require(listFile)

import 'common/Extend'
import { AllChains, IChain } from 'chain/allchain'
import { EventBatchSvr } from 'service/event.batch.service'
import { IEventCfg } from 'interface/IEventCfg'
import { ZRedisClient } from 'zutils'

let svrs: any[] = []
let lock = false

async function initEventSvrs() {
  const cfgMap: Map<IChain, IEventCfg[]> = new Map()
  for (let cfg of events) {
    cfg.address = cfg.address.toLowerCase()
    cfg.abi = require(`../config/event_abis/${cfg.abi}.json`)
    const chainCfg = AllChains.find(chain => chain.id === cfg.chain)
    if (!chainCfg) {
      logger.error('chainCfg not found: ' + cfg.chain)
      process.exit(1)
    }
    if (!cfgMap.has(chainCfg)) {
      cfgMap.set(chainCfg, [])
    }
    cfgMap.get(chainCfg)?.push(cfg)
  }
  for (let chainCfg of cfgMap.keys()) {
    const svr = new EventBatchSvr(chainCfg, cfgMap.get(chainCfg)!)
    svrs.push(svr)
  }
}

async function parseAllEvents() {
  if (lock) {
    logger.warn('sync in process, cancel.')
    return
  }
  lock = true
  logger.info('begin sync events: ' + svrs.map(svr => svr.chainCfg.id).join(','))
  for (let svr of svrs) {
    try {
      await svr.execute()
    } catch (err) {
      logger.info('sync events  error:: chain: ' + svr.chainCfg.id)
      logger.info(err)
    }
  }
  lock = false
}

;(async () => {
  let opts = { url: process.env.REDIS }
  new ZRedisClient(opts)
  logger.info('REDIS Connected')
  await initEventSvrs()
  setInterval(function () {
    parseAllEvents()
  }, 10000)
  parseAllEvents()
})()
