import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'
import { TokenHolder } from './TokenHolder'

@dbconn()
@index({ address: 1 }, { unique: false })
@index({ chain: 1, hash: 1, logIndex: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'ft_transfer_event', timestamps: true },
})
export class FtTransferEventClass extends BaseModule {
  @prop({ required: true })
  public address!: string
  @prop({ required: true })
  public chain: string
  @prop({ required: true })
  public logIndex: number
  @prop()
  public event: string
  // event hash
  @prop({ required: true })
  public hash: string
  @prop()
  public blockNumber: number
  @prop()
  public blockHash: string
  @prop()
  public removed: boolean

  @prop()
  public from: string
  @prop()
  public to: string
  @prop()
  public amount: string

  @prop()
  public blockTime: number
  @prop({ default: 0 })
  public version: number

  public static async saveEvent(event: any) {
    const amount = event.returnValues?.value
    if (amount == undefined) {
      return
    }

    const logIndex = parseInt(event.logIndex || '0')
    const from = event.decodedData.from.toLowerCase()
    const to = event.decodedData.to.toLowerCase()
    const hash = event.hash || event.transactionHash
    const data = {
      address: event.address.toLowerCase(),
      blockNumber: parseInt(event.blockNumber),
      removed: event.removed,
      from,
      to,
      amount,
      $inc: { version: 1 },
    }

    let record = await FtTransferEvent.insertOrUpdate({ hash, logIndex, chain: event.chain }, data)
    await TokenHolder.saveData(record)
  }
}

export const FtTransferEvent = getModelForClass(FtTransferEventClass, {
  existingConnection: FtTransferEventClass['db'],
})
