import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'
import { GeneralEvent } from './GeneralEvent'
import { ZERO_ADDRESS } from 'common/Constants'

@dbconn()
@index({ chain: 1, address: 1, user: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'token_holder', timestamps: true },
})
export class TokenHolderClass extends BaseModule {
  @prop({ required: true })
  public chain: string

  @prop({ required: true })
  public address!: string

  @prop()
  public user: string

  @prop({ required: true })
  public amount: string

  @prop()
  public blockNumber: number

  public static async parseEvent(event: typeof GeneralEvent) {
    const address = event.address.toLowerCase()
    const chain = event.chain
    const blockNumber = event.blockNumber
    let { from, to, value } = event.decodedData
    from = from.toLowerCase()
    to = to.toLowerCase()
    if (from !== ZERO_ADDRESS) {
      let record = await TokenHolder.findOne({ address, chain, user: from })
      if (record) {
        record.amount = BigInt(record.amount) - BigInt(value)
        record.blockNumber = blockNumber
      }
      await record.save()
    }
    if (to !== ZERO_ADDRESS) {
      let record = await TokenHolder.findOne({ address, chain, user: to })
      if (!record) {
        record = new TokenHolder({ address, chain, user: to, amount: BigInt(value), blockNumber })
      } else {
        record.amount = BigInt(record.amount) + BigInt(value)
        record.blockNumber = blockNumber
      }
      await record.save()
    }
  }
}

export const TokenHolder = getModelForClass(TokenHolderClass, {
  existingConnection: TokenHolderClass['db'],
})
