import * as dotenv from 'dotenv'
import logger from 'logger/logger'
const envFile = process.env.NODE_ENV && process.env.NODE_ENV === 'production' ? `.env.production` : '.env.development'
dotenv.config({ path: envFile })
import { EventSyncSvr } from 'service/event.sync.service'
import { NftTransferEvent } from 'models/NftTransferEvent'
import { FtTransferEvent } from 'models/FtTransferEvent'

import 'common/Extend'
import { ZRedisClient } from 'zutils'

let svrs: any[] = []
let lock = false

let eventProcessers = {
  NftTransferEvent: NftTransferEvent,
  FtTransferEvent: FtTransferEvent,
}

const events = require('../config/events.json')

async function initEventSvrs() {
  for (let event of events) {
    let eventSvr = new EventSyncSvr({
      address: event.address,
      event: event.event,
      abi: require('abis/' + event.abi + '.json').abi,
      fromBlock: event.fromBlock,
      eventProcesser: eventProcessers[event.eventProcesser],
    })
    svrs.push(eventSvr)
  }
}

async function parseAllEvents() {
  if (lock) {
    logger.info('sync in process, cancel.')
    return
  }
  lock = true
  logger.info('begin sync nft events: ' + svrs.length)
  for (let svr of svrs) {
    try {
      await svr.parseEvents()
    } catch (err) {
      logger.info('parse event with error:: address: ' + svr.address + ' event: ' + svr.event)
      logger.info(err)
    }
  }
  lock = false
}

async function main() {
  let opts = { url: process.env.REDIS }
  new ZRedisClient(opts)
  logger.info('REDIS Connected')
  await initEventSvrs()
  setInterval(function () {
    parseAllEvents()
  }, 60000)
  parseAllEvents()
}

main()
